@@ -38,9 +38,9 @@
 #include "winrtcommon.h"
 
 #include <QtSensors/QCompass>
+#include <private/qeventdispatcher_winrt_p.h>
 
-QT_USE_NAMESPACE
-
+#include <functional>
 #include <wrl.h>
 #include <windows.devices.sensors.h>
 using namespace Microsoft::WRL;
@@ -133,20 +133,24 @@ WinRtCompass::WinRtCompass(QSensor *sensor)
     : QSensorBackend(sensor), d_ptr(new WinRtCompassPrivate(this))
 {
     Q_D(WinRtCompass);
-    HStringReference classId(RuntimeClass_Windows_Devices_Sensors_Compass);
-    ComPtr<ICompassStatics> factory;
-    HRESULT hr = RoGetActivationFactory(classId.Get(), IID_PPV_ARGS(&factory));
-    if (FAILED(hr)) {
-        qCWarning(lcWinRtSensors) << "Unable to initialize light sensor factory."
-                                  << qt_error_string(hr);
-        sensorError(hr);
-        return;
-    }
+    HRESULT hr = QEventDispatcherWinRT::runOnXamlThread([d]() {
+        HStringReference classId(RuntimeClass_Windows_Devices_Sensors_Compass);
+        ComPtr<ICompassStatics> factory;
+        HRESULT hr = RoGetActivationFactory(classId.Get(), IID_PPV_ARGS(&factory));
+        if (FAILED(hr)) {
+            qCWarning(lcWinRtSensors) << "Unable to initialize light sensor factory."
+                                      << qt_error_string(hr);
+            return hr;
+        }
 
-    hr = factory->GetDefault(&d->sensor);
+        hr = factory->GetDefault(&d->sensor);
+        if (FAILED(hr)) {
+            qCWarning(lcWinRtSensors) << "Unable to get default compass."
+                                      << qt_error_string(hr);
+        }
+        return hr;
+    });
     if (FAILED(hr) || !d->sensor) {
-        qCWarning(lcWinRtSensors) << "Unable to get default compass."
-                                  << qt_error_string(hr);
         sensorError(hr);
         return;
     }
@@ -177,9 +181,11 @@ void WinRtCompass::start()
     if (d->token.value)
         return;
 
-    ComPtr<CompassReadingHandler> callback =
+    HRESULT hr = QEventDispatcherWinRT::runOnXamlThread([d]() {
+        ComPtr<CompassReadingHandler> callback =
             Callback<CompassReadingHandler>(d, &WinRtCompassPrivate::readingChanged);
-    HRESULT hr = d->sensor->add_ReadingChanged(callback.Get(), &d->token);
+        return d->sensor->add_ReadingChanged(callback.Get(), &d->token);
+    });
     if (FAILED(hr)) {
         qCWarning(lcWinRtSensors) << "Unable to attach to reading changed event."
                                   << qt_error_string(hr);
@@ -207,15 +213,16 @@ void WinRtCompass::stop()
         return;
     if (!d->token.value)
         return;
-
-    HRESULT hr = d->sensor->remove_ReadingChanged(d->token);
+    HRESULT hr = QEventDispatcherWinRT::runOnXamlThread([d]() {
+        return d->sensor->remove_ReadingChanged(d->token);
+    });
     if (FAILED(hr)) {
         qCWarning(lcWinRtSensors) << "Unable to detach from reading changed event."
                                   << qt_error_string(hr);
         sensorError(hr);
         return;
     }
-    d->sensor->put_ReportInterval(0);
+    hr = d->sensor->put_ReportInterval(0);
     if (FAILED(hr)) {
         qCWarning(lcWinRtSensors) << "Unable to reset report interval."
                                   << qt_error_string(hr);
